package org.ananas.mac.menu;

/*
 * Copyright 2003, Benoit Marchal. All rights reserved.
 * Distributed under the artistic license (open source).
 * See www.ananas.org for licensing information.
 */

import java.awt.Menu;
import javax.swing.Icon;
import javax.swing.JMenu;
import java.awt.MenuShortcut;
import javax.swing.KeyStroke;
import java.awt.event.ActionListener;

/**
 * <p>A system-independent interface for "application-smart" menu items.</p>
 * <p>An "application-smart" menu item is a menu item that will appear in
 * in the application menu under MacOS. It will be managed as a regular
 * menu under other OS.</p>
 * <p>The following code sample creates a "Quit" menu, registers it on
 * the file menu and adds an action listener:</p><hr>
 * <pre> ApplicationItem about = ApplicationItemFactory.createQuitItem();
 * quit.register("Exit",fileMenu);
 * quit.addActionListener(new ActionListener()
 * {
 *    public void actionPerformed(ActionEvent e)
 *    {
 *       doQuit();
 *    }
 * });</pre><hr>
 * <p><b>Important</b> you have to register with a menu, even on MacOS. The
 * library automatically decides if the item should be added to a regular menu
 * or to the application menu. Your code remains platform independent</p>
 * <p>The register() method is heavily overloaded to support AWT and Swing.</p>
 *
 * @author <a href="http://www.marchal.com">Beno&icirc;t Marchal</a>
 * @see org.ananas.mac.menu.ApplicationItemFactory
 * @version 1.0
 */
public interface ApplicationItem
{
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param label item label
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(String label,Menu menu)
      throws IllegalStateException;
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param label item label
    * @param shortcut menu item shortcut
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(String label,MenuShortcut shortcut,Menu menu)
      throws IllegalStateException;
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param label item label
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(String label,JMenu menu)
      throws IllegalStateException;
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param icon item icon
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(Icon icon,JMenu menu)
      throws IllegalStateException;
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param label item label
    * @param icon item icon
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(String label,Icon icon,JMenu menu)
      throws IllegalStateException;
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param label item label
    * @param mnemonic menu item mnemonic (underlined letter)
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(String label,int mnemonic,JMenu menu)
      throws IllegalStateException;
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param label item label
    * @param accelerator accelerator (shortcut) for the menu
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(String label,KeyStroke accelerator,JMenu menu)
      throws IllegalStateException;
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param icon item icon
    * @param accelerator accelerator (shortcut) for the menu
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(Icon icon,KeyStroke accelerator,JMenu menu)
      throws IllegalStateException;
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param label item label
    * @param icon item icon
    * @param accelerator accelerator (shortcut) for the menu
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(String label,Icon icon,KeyStroke accelerator,JMenu menu)
      throws IllegalStateException;
   /**
    * Registers the "application" menu item. On MacOS, this method ignores
    * its parameters and registers the menu in the application menu. On other
    * OSes, it creates a regular menu item in the menu.
    * @param label item label
    * @param mnemonic menu item mnemonic (underlined letter)
    * @param accelerator accelerator (shortcut) for the menu
    * @param menu menu to add the item to
    * @throws IllegalStateException if calling register() twice or more
    */
   public void register(String label,int mnemonic,KeyStroke accelerator,JMenu menu)
      throws IllegalStateException;
   /**
    * Adds the specified action listener to receive action events from this menu
    * item. Action events occur when a user selects the menu, regardless if the menu
    * is in the application menu or in the menu bar.
    * @param listener the action listener
    */
   public void addActionListener(ActionListener listener)
      throws IllegalStateException;
   /**
    * Removes the specified action listener so it no longer receives action events
    * from this menu item.
    * @param listener the action listener
    */
   public void removeActionListener(ActionListener listener)
      throws IllegalStateException;
   /**
    * <p>Returns true if the menu item will or has been added to application menu.
    * This is useful if you need to change/adapt the menu bar depending on
    * whether the menu is in the application menu.</p>
    * <p>The following code sample adds a menu separator if the "Quit" menu
    * will go in the menu bar.</p><hr>
    * <pre>if(quit.isAddedToMenu())
    *    fileMenu.addSeparator();</pre><hr>
    * <p>You can call this method before or after calling register()</p>
    * @return true if the menu item is added to the regular menu during registration,
    *    false if it is added to the application menu
    */
   public boolean isAddedToMenu();
}