/**
 * DirectoryReader returns the content of a directory as SAX events.
 * It effectively reads the directory
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.xm;

import java.io.*;
import org.xml.sax.*;
import java.util.Date;
import org.xml.sax.helpers.*;
import java.text.SimpleDateFormat;
import org.ananas.util.NotImplementedException;

public class DirectoryReader
{
   /**
    * date/time formatter for XML Schema, part 2 lexical representation
    * or CCYY-MM-DDThh:mm:ss (e.g. 1999-05-31T13:20:00-05:00)
    */
   protected final static SimpleDateFormat dateTimeFormat =
      new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

   /**
    * namespace URI for DirectoryReader
    */
   public final static String namespaceURI =
      "http://www.ananas.org/2001/XM/Walk/Directory";

   /**
    * SAX ContentHandler to generate events to
    */
   protected ContentHandler contentHandler = null;

   /**
    * output messages on this messenger
    */
   protected Messenger messenger = null;

   /**
    * if true, the reader is "embedded" in another stream so it
    * should refrain from issuing start/endDocument()
    */
   protected boolean embedded;

   /**
    * creates a new DirectoryReader
    * @param messenger messenger to report progress and error messages to
    * @param embedded if true, the reader is embedded in another
    *    stream so it should not issue start/endDocument, otherwise
    *    the reader creates a complete document
    */
   public DirectoryReader(Messenger messenger,boolean embedded)
   {
      this.messenger = messenger;
      this.embedded = embedded;
   }

   /**
    * creates a new DirectoryReader, assumes the reader is not
    * embedded
    * @param messenger messenger to report progress and error messages to
    */
   public DirectoryReader(Messenger messenger)
   {
      this(messenger,false);
   }

   /**
    * set a new ContentHandler for the reader to reports too
    * @param contentHandler new ContentHandler
    */
   public void setContentHandler(ContentHandler contentHandler)
   {
      this.contentHandler = contentHandler;
   }

   /**
    * returns the current ContentHandler
    * @return current ContentHandler
    */
   public ContentHandler getContentHandler()
   {
      return contentHandler;
   }
   /**
    * reads the given directory
    * @param dir directory to read
    * @param toMark a file to mark (typically the .xm file),
    *    may be null
    * @throw XMException dir is not a directory
    */
   public void read(File dir,File toMark)
      throws XMException
   {
      try
      {
         if(toMark != null)
            toMark = toMark.getCanonicalFile();
         if(contentHandler == null)
            return;
         if(!dir.isDirectory())
            messenger.fatal(new XMException(Resources.getString("notdirectory"),new Object[] { dir.getPath() }));
         if(!embedded)
            contentHandler.startDocument();
         contentHandler.startPrefixMapping("xm",namespaceURI);
         AttributesImpl attributes = new AttributesImpl();
         attributes.addAttribute("","xm","xmlns:xm","CDATA",namespaceURI);
         contentHandler.startElement(namespaceURI,"Directory","xm:Directory",attributes);
         attributes.clear();
         File[] files = dir.listFiles();
         File file = null;
         char[] chars = null;
         for(int i = 0;i < files.length;i++)
         {
            file = files[i];
            attributes.addAttribute("","isDirectory","isDirectory","CDATA",String.valueOf(file.isDirectory()));
            attributes.addAttribute("","isFile","isFile","CDATA",String.valueOf(file.isFile()));
            attributes.addAttribute("","isHidden","isHidden","CDATA",String.valueOf(file.isHidden()));
            if(toMark != null)
            {
               // use the canonical file for comparison: the two files
               // have been constructed differently and Java may report
               // them as different even though they point to the same file
               File canonicalFile = file.getCanonicalFile();
               attributes.addAttribute("","isMarked","isMarked","CDATA",String.valueOf(canonicalFile.equals(toMark)));
            }
            else
               attributes.addAttribute("","isMarked","isMarked","CDATA","false");
            attributes.addAttribute("","canRead","canRead","CDATA",String.valueOf(file.canRead()));
            attributes.addAttribute("","canWrite","canWrite","CDATA",String.valueOf(file.canWrite()));
            attributes.addAttribute("","length","length","CDATA",String.valueOf(file.length()));
            attributes.addAttribute("","lastModified","lastModified","CDATA",dateTimeFormat.format(new Date(file.lastModified())));
            contentHandler.startElement(namespaceURI,"File","xm:File",attributes);
            attributes.clear();
            chars = file.getName().toCharArray();
            contentHandler.characters(chars,0,chars.length);
            contentHandler.endElement(namespaceURI,"File","xm:File");
         }
         contentHandler.endElement(namespaceURI,"Directory","xm:Directory");
         contentHandler.endPrefixMapping("xm");
         if(!embedded)
            contentHandler.endDocument();
      }
      catch(IOException e)
      {
         throw new XMException(e);
      }
      catch(SAXException e)
      {
         throw new XMException(e);
      }
   }

   /**
    * entry-point, as a testing application
    * @param args command-line arguments
    */
   public static void main(String[] args)
      throws IOException, XMException,
             javax.xml.transform.TransformerConfigurationException
   {
      OutputStream out = System.out;
      if(args.length > 1)
         out = new FileOutputStream(args[1]);
      DirectoryReader reader = new DirectoryReader(new DefaultMessenger());
      javax.xml.transform.TransformerFactory transformerFactory =
         javax.xml.transform.TransformerFactory.newInstance();
      if(transformerFactory.getFeature(javax.xml.transform.sax.SAXTransformerFactory.FEATURE))
      {
         javax.xml.transform.sax.SAXTransformerFactory saxTransformerFactory =
            (javax.xml.transform.sax.SAXTransformerFactory)transformerFactory;
         javax.xml.transform.sax.TransformerHandler identityHandler =
            saxTransformerFactory.newTransformerHandler();
         identityHandler.setResult(new javax.xml.transform.stream.StreamResult(out));
         reader.setContentHandler(identityHandler);
         reader.read(new File(args[0]),null);
         if(out instanceof FileOutputStream)
            out.close();
      }
      else
         throw new NotImplementedException("TransformerFactory must support SAXTransformerFactory");
   }
}
