/**
 * DirectoryWalker runs through the source directory and apply the
 * styling as appropriate to create the target directory.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.xm;

import java.io.*;
import java.util.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import org.ananas.util.NotImplementedException;

public class DirectoryWalker
{
   /**
    * outputs messages on this messenger
    */
   protected Messenger messenger = null;

   /**
    * selects the right mover for the job
    */
   protected MoversSupervisor supervisor;

   /**
    * creates a new DirectoryWalker
    * @param messenger messenger to report progress and error messages to
    * @param rulesDir directory to look XSLT files in
    * @param build does a build, i.e. process all files
    */
   public DirectoryWalker(Messenger messenger,
                          String rulesDir,
                          boolean build)
      throws IOException, XMException
   {
      this(messenger,new File(rulesDir),build);
   }

   /**
    * creates a new DirectoryWalker
    * @param messenger messenger to report progress and error messages to
    * @param rulesDir directory to look XSLT files in
    * @param build does a build, i.e. process all files
    */
   public DirectoryWalker(Messenger messenger,
                          File rulesDir,
                          boolean build)
      throws XMException
   {
      this.messenger = messenger;
      TemplatesManager templates =
         new TemplatesManager(messenger,rulesDir);
      supervisor = new MoversSupervisor(messenger,templates,build);
   }

   /**
    * walks through the source directory, processing files as it
    * goes along to create the target directory
    * @param source source directory
    * @param target target directory
    * @throws IOException error with the file system
    * @throws XMException error caught by the application
    */
   public void walk(String source,String target)
      throws IOException, XMException
   {
   	  try
   	  {
         messenger.begin(source,target);
         walk(new File(source),new File(target),0);
   	  }
   	  finally
   	  {
         messenger.end();
   	  }
   }

   /**
    * actual implementation of the walking
    * it takes one extra parameter, depth, to record how far
    * we have been in the file system
    * @param source source directory
    * @param target target directory
    * @param depth an indicator of how far we are in the file system
    * @throws IOException error with the file system
    * @throws XMException error caught by the application
    * @return true if walking should continue, false if the messenger
    *    has asked for the end of the walking
    */
   protected boolean walk(File source,File target,int depth)
      throws IOException, XMException
   {
      if(source.isDirectory())
      {
         File[] files = source.listFiles();
         if(files == null)
         {
            messenger.fatal(new XMException(Resources.getString("notdirectory"),
                                            new Object[] {source.getAbsolutePath()}));
            return true;
         }
         File[] dirs = new File[files.length],
                docs = new File[files.length];
         int idirs = 0,
             idocs = 0;
         for(int i = 0;i < files.length;i++)
         {
            if(files[i].isDirectory())
               dirs[idirs++] = files[i];
            else if(files[i].isFile())
               docs[idocs++] = files[i];
            else
               throw new NotImplementedException("Expecting file or directory");
         }
         if(!(target.exists() && target.isDirectory()))
            if(!target.mkdirs())
               messenger.fatal(new XMException(Resources.getString("cannotcreatedirectory"),
                                               new Object[] {target.getAbsolutePath()}));
         for(int i = 0;i < idocs;i++)
         {
            Mover mover = supervisor.getMover(docs[i]);
            File result = mover.move(docs[i],target,depth);
            if(!messenger.progress(docs[i],result))
               return false;
         }
         for(int i = 0;i < idirs;i++)
            if(!walk(dirs[i],new File(target,dirs[i].getName()),depth + 1))
               return false;
      }
      else
         messenger.fatal(new XMException(Resources.getString("notdirectory"),
                                         new Object[] {source.getAbsolutePath()}));
      return true;
   }
}
