/**
 * This interface defines methods used by XM to report errors, warnings
 * and other information messages. UI should implement this interface
 * to report messages to the user.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.xm;

import java.io.File;

public interface Messenger
{
   /**
    * Non-fatal error message.
    * @param e exception describing the error.
    */
   public void error(XMException e)
      throws XMException;

   /**
    * Fatal error message.
    * @param e exception describing the error.
    */
   public void fatal(XMException e)
      throws XMException;

   /**
    * Warning message. This is not an error.
    * @param e exception describing the warning.
    */
   public void warning(XMException e)
      throws XMException;

   /**
    * Progress indicator: a file has been processed. If no result
    * file was created, resultFile is null. It gives the user
    * a chance to interrupt processing through its return value.
    * @param sourceFile the file just processed
    * @param resultFile the file just created or null if no result
    *   (e.g. the result file already exists)
    * @return true if the caller should proceed to the next file, false
    *    if the user has requested that processing stops
    */
   public boolean progress(File sourceFile,File resultFile)
      throws XMException;

   /**
    * Catch-all for other messages. UI should display this to the user.
    * @param msg information message
    */
   public void info(String msg)
      throws XMException;

   /**
    * Catch-all for other messages. UI should display this to the user.
    * This is for convenience: it is given the error message in a format
    * compatible with MessageFormat.format()
    * @param pattern message pattern
    * @param arguments message arguments
    * @see java.text.MessageFormat
    */
   public void info(String pattern,Object[] arguments)
      throws XMException;

   /**
    * Lifecycle: called by DirectoryWalker before styling a site.
    * @param source directory that will be styled
    * @param target directory where the result was placed
    */
   public void begin(String source,String target)
      throws XMException;

   /**
    * Lifecycle: called by DirectoryWalker when the site has been processed
    * completely.
    */
   public void end()
      throws XMException;
}
