/**
 * ProcessingInstructionHandler parses a document to extract
 * the <?xml-stylesheet?> and <?xm-xsl-param?> processing instructions.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.xm;

import java.util.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

public class ProcessingInstructionHandler
   extends DefaultHandler
{
   /**
    * PseudoAttributeTokenizer tokenizes (and mostly parses)
    * the pseudo-attribute grammar defined in <?xml-stylesheet?>
    * recommendation
    */
   protected class PseudoAttributeTokenizer
   {
      /**
       * current position in the data
       */
      protected int pos = 0,

      /**
       * length of the buffer, speeds comparison
       */
                    length;

      /**
       * the token being read is stored here, it saves us
       * reallocating a buffer with every call
       */
      protected StringBuffer token = new StringBuffer();

      /**
       * data buffer to tokenize
       */
      protected String buffer;

      /**
       * if set this locator is used to report errors
       */
      protected Locator locator;

      /**
       * error messages
       */
      protected static final String
         UNEXPECTED_EOS = "Unexpected end of data in processing instruction",
         UNKNOWN_ENTITY = "Unknown entity in processing instruction",
         EXPECTED_EQUAL = "Expecting '=' in processing instruction",
         EXPECTED_QUOTE = "Expecting '\"' or \"'\" in processing instruction";

      /**
       * creates a PseudoAttributeTokenizer
       * @param buffer data buffer to tokenizer
       * @param locator locator where the data buffer was found
       *    useful when reporting errors
       */
      public PseudoAttributeTokenizer(String buffer,Locator locator)
      {
         this.locator = locator;
         this.buffer = buffer;
         // small optimization, we do a lot of comparisons
         length = buffer.length();
      }

      /**
       * unread a character, making it available for the next call
       * to getc(), given we operator with an in-memory buffer,
       * this is slightly easier to read than a lookahead character
       */
      protected void putc()
      {
         pos--;
      }

      /**
       * read the next character in the data buffer
       * @return the next character or -1 if at the end of the buffer
       */
      protected int getc()
      {
         if(pos < length)
            return buffer.charAt(pos++);
         else
            return -1;
      }

      /**
       * helper method, equivallent to getc() but bypasses all the
       * spaces
       * @return the next character (after bypassing spaces) or -1
       *    if at the end of the buffer
       */
      protected int eatSpaces()
      {
         int c = getc();
         while((c != -1) && Character.isWhitespace((char)c))
            c = getc();
         return c;
      }

      /**
       * reads an entity content (the & must have already been
       * read) and returns the corresponding character
       * @return the character this entity escapes
       */
      protected char eatEntity()
         throws SAXParseException
      {
         StringBuffer entityToken = new StringBuffer();
         int c = getc();
         for(;;)
            if(c == -1)
               throw new SAXParseException(UNEXPECTED_EOS,locator);
            else if(c == ';')
            {
               String entity = entityToken.toString();
               if(entity.equals("amp"))
                  return '&';
               else if(entity.equals("lt"))
                  return '<';
               else if(entity.equals("gt"))
                  return '>';
               else if(entity.equals("quot"))
                  return '"';
               else if(entity.equals("apos"))
                  return '\'';
               else
                  throw new SAXParseException(UNKNOWN_ENTITY,locator);
            }
            else
            {
               entityToken.append((char)c);
               c = getc();
            }
      }

      /**
       * returns true if there are more tokens, if it returns true,
       * one knows that nextName() and nextValue() will return
       * a value (unless there's a syntax error)
       * @return true if there are more tokens, false otherwise
       */
      public boolean hasMoreTokens()
      {
         int c = eatSpaces();
         if(c != -1)
         {
            putc();
            return true;
         }
         else
            return false;
      }

      /**
       * returns the next pseudo-attribute name from the buffer
       * must be followed by a call to nextValue() (i.e. it is
       * illegal to call nextName() twice in a row
       * @return the next name
       */
      public String nextName()
         throws SAXParseException
      {
         token.setLength(0);
         int c = eatSpaces();
         for(;;)
            if(c == -1)
               throw new SAXParseException(UNEXPECTED_EOS,locator);
            // strictly speaking a name cannot start with a digit...
            else if(!Character.isLetterOrDigit((char)c) && c != '-')
            {
               putc();   // put it back for the next call
               return token.length() == 0 ? null : token.toString();
            }
            else
            {
               token.append((char)c);
               c = getc();
            }
      }

      /**
       * returns the next pseudo-attribute value from the buffer
       * must be followed by a call to nextName() (i.e. it is
       * illegal to call nextValue() twice in a row
       * @return the next value
       */
      public String nextValue()
         throws SAXParseException
      {
         token.setLength(0);
         int c = eatSpaces();
         if(c == -1 || c != '=')
            throw new SAXParseException(EXPECTED_EQUAL,locator);
         c = eatSpaces();
         if(c == -1 || (c != '"' && c != '\''))
            throw new SAXParseException(EXPECTED_QUOTE,locator);
         char endString = (char)c;
         c = getc();
         for(;;)
            if(c == -1)
               throw new SAXParseException(UNEXPECTED_EOS,locator);
            else if(c == endString)
               // not putting it back, we need to consume it
               return token.length() == 0 ? null : token.toString();
            else if(c == '&')
            {
               token.append(eatEntity());
               c = getc();
            }
            else
            {
               token.append((char)c);
               c = getc();
            }
      }
   }

   /**
    * stylesheet href
    */
   protected String href;

   /**
    * whether to load the external DTD or not
    */
   protected boolean loadExternalDTD;

   /**
    * indicates whether we should read parameters or not
    * this is set to false when we hit an alternate <?xml-stylesheet?>
    */
   protected boolean readParams;

   /**
    * the parameters associated with the href style sheet or the
    * default style sheet (if href is null)
    */
   protected Map params = new HashMap();

   /**
    * stores the parser Locator, useful when reporting errors
    */
   protected Locator locator = null;

   /**
    * this special exception is thrown to indicate we have found
    * the end of the prologue, by comparing against it, we know it's
    * not an error
    */
   protected SAXException foundException = new SAXException("End of prologue found");

   /**
    * parser gives us a Locator to use for error reporting
    * @param locator the locator
    */
   public void setDocumentLocator(Locator locator)
   {
      this.locator = locator;
   }

   /**
    * signals the beginning of the document
    */
   public void startDocument()
   {
      href = null;
      readParams = true;
      loadExternalDTD = true;
      params.clear();
   }

   /**
    * a processing instruction has been found, decode it
    * @param target processing instruction target
    * @data content of the processing instruction
    */
   public void processingInstruction(String target,String data)
      throws SAXException
   {
      if(target.equals("xml-stylesheet"))
      {
         String href = null,
                type = null;
         boolean alternate = false;
         PseudoAttributeTokenizer tokenizer =
            new PseudoAttributeTokenizer(data,locator);
         while(tokenizer.hasMoreTokens())
         {
            String name = tokenizer.nextName(),
                   value = tokenizer.nextValue();
            if(name.equals("href"))
               href = value;
            else if(name.equals("alternate"))
               alternate = value.equals("yes");
            else if(name.equals("type"))
               type = value.trim();
            // ignore the media attribute...
         }
         if(type != null && href != null && !alternate &&
            (type.equals("text/xsl") || type.equals("text/xml") ||
             type.equals("application/xml+xslt")))
         {
            this.href = href;
            params.clear();
            readParams = true;
         }
         else
            readParams = false;
      }
      else if(target.equals("xm-xsl-param") && readParams)
      {
         PseudoAttributeTokenizer tokenizer =
            new PseudoAttributeTokenizer(data,locator);
         String attrName = null,
                attrValue = null;
         while(tokenizer.hasMoreTokens())
         {
            String name = tokenizer.nextName(),
                   value = tokenizer.nextValue();
            if(name.equals("name"))
               attrName = value;
            else if(name.equals("value"))
               attrValue = value;
         }
         if(attrName != null && attrValue != null)
            params.put(attrName,attrValue);
         else
            // BUGBUG -- should come from a Locale message
            throw new SAXParseException("Both name and value attributes are required in xm-xsl-param",locator);
      }
      else if(target.equals("xm-pragma"))
      {
         PseudoAttributeTokenizer tokenizer =
            new PseudoAttributeTokenizer(data,locator);
         while(tokenizer.hasMoreTokens())
         {
            String name = tokenizer.nextName(),
                   value = tokenizer.nextValue();
            if(name.equals("load-external-dtd"))
               loadExternalDTD = value.equals("yes");
         }
      }
   }

   /**
    * signals the beginning of an element, knowing we are past
    * the prolog, we throw the foundException
    * @throws SAXException to stop parsing after the prologue
    */
   public void startElement(String namespaceURI,
                            String localName,
                            String qName,
                            Attributes atts)
      throws SAXException
   {
      throw foundException;
   }

   /**
    * retrieves the HRef property
    * @return style sheet URI or null if using the default style sheet
    */
   public String getHRef()
   {
      return href;
   }

   /**
    * retrieves the LoadExternalDTD property
    * @return true if a pragma set it to true, false otherwise
    */
   public boolean isLoadExternalDTD()
   {
      return loadExternalDTD;
   }

   /**
    * retrieves the params property
    * @return a map with the list of params
    */
   public Map getParams()
   {
      return params;
   }

   /**
    * convenience: returns an iterator over the list of params
    * @return params iterator
    */
   public Iterator getParamsIterator()
   {
      return params.entrySet().iterator();
   }

   /**
    * tests if a SAXException is the stop exception, the exception
    * thrown after the end of the prologue, thereby telling us if
    * the parsing went well or not; arguably it's bizarre to throw
    * an exception to signal that parsing completed satisfactorily
    * and that we are past the prologue but there's no other mechanism
    * to stop a SAX parser
    * @param e exception to test
    * @return true if e is the stop exception, false otherwise
    */
   public boolean isStopException(SAXException e)
   {
      return e == foundException;
   }
}
