/**
 * TemplatesManager loads the templates, it is responsible
 * to cache as much of the templates as possible.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

package org.ananas.xm;

import java.io.*;
import java.util.*;
import org.xml.sax.*;
import org.ananas.util.*;
import javax.xml.transform.*;
import javax.xml.transform.sax.*;
import javax.xml.transform.stream.StreamSource;

public class TemplatesManager
{
   /**
    * ReferenceResolver implements XM special rules when it comes
    * to loading XSLT style sheets, specifically it loads the
    * style sheets from a special directory the "rules" directory
    */
   protected class ReferenceResolver
      implements URIResolver
   {
      /**
       * the directory which should contain the style sheets
       */
      protected File rulesDir,

      /**
       * the directory which should contain the document (may be null)
       */
                     docsDir;

      /**
       * Creates a ReferenceResolver
       * @param rulesDir directory for the style sheets
       */
      public ReferenceResolver(File rulesDir)
      {
         this.rulesDir = rulesDir;
      }

      /**
       * sets the document directory
       * @param docsDir the new document directory
       */
      public void setDocsDir(File docsDir)
      {
         this.docsDir = docsDir;
      }

      /**
       * attempts to load style sheets (and style sheets only)
       * from the rules directory, expects style sheets to end
       * in .xsl
       * @param href URI to style sheet to load
       * @param base base directory to look into (ignored)
       * @return a Source object pointing to the style sheet
       *    or null if the style sheet cannot be found
       */
      public Source resolve(String href,String base)
         throws TransformerException
      {
         if(href.endsWith(".xsl"))
         {
            File file = new File(rulesDir,href);
            if(file.exists())
               return new StreamSource(file);
         }
         else if(href.startsWith("x-source:") && docsDir != null)
         {
            File file = new File(docsDir,href.substring(9));
            if(file.exists())
               // BUGBUG -- this is only an expediant to let it load
               // any document, with or without DTDs a better solution
               // is to use an XMLReader that knows how to decode
               // the pragma directives
               try
               {
                  return new SAXSource(JAXPHelper.createXMLReader(true,false),
                                       JAXPHelper.toInputSource(file));
               }
               catch(SAXException e)
               {
                  throw new TransformerException(e);
               }
               catch(IOException e)
               {
                  throw new TransformerException(e);
               }
            else
               return new StreamSource(new StringReader("<xm:NotAvailable xmlns:xm='http://www.ananas.org/2001/XM/Walk/Directory'/>"),"file:null");
         }
         return null;
      }
   }

   /**
    * Messenger to report errors
    */
   protected Messenger messenger;

   /**
    * "compiled" style sheets
    */
   protected Map templatesMap = new HashMap();

   /**
    * ReferenceResolver to use to load style sheets
    */
   protected ReferenceResolver referenceResolver;

   /**
    * transformer factory to create new Templates
    */
   protected SAXTransformerFactory factory;

   /**
    * Ceates a new TemplatesManager.
    * @param messenger Messenger to report errors to
    * @param rulesDir stylesheets directory
    */
   public TemplatesManager(Messenger messenger,File rulesDir)
      throws XMException
   {
      this.messenger = messenger;
      referenceResolver = new ReferenceResolver(rulesDir);
      try
      {
         TransformerFactory transformerFactory = TransformerFactory.newInstance();
         if(transformerFactory.getFeature(SAXTransformerFactory.FEATURE))
            factory = (SAXTransformerFactory)transformerFactory;
         else
            throw new NotImplementedException("Processor does not support SAXTransformerFactory");
         // loads the default style sheet
         getTemplatesHelper("default.xsl",new File(rulesDir,"default.xsl"));
      }
      catch(TransformerFactoryConfigurationError e)
      {
         messenger.fatal(new XMException("TransformerFactoryConfigurationError: " + e.getMessage()));
      }
   }

   /**
    * returns the Templates object identified by the stylesheet URI
    * whenever possible, it accesses the style sheet from the cache
    * @param stylesheet URI for the style sheet, may be null (to
    *    access the default style sheet)
    * @param sourceFile directory to look into if the style sheet
    *    is not in the rules directory
    * @return "compiled" form of the style sheet
    */
   public Templates getTemplates(String stylesheet,
                                 File sourceFile)
      throws XMException
   {
      return stylesheet != null ? getTemplatesHelper(stylesheet,sourceFile) :
                                  getDefaultTemplates();
   }

   /**
    * helper method, takes care of loading the Templates, if required
    * returns the Templates object identified by the stylesheet URI
    * whenever possible, it accesses the style sheet from the cache
    * @param stylesheet URI for the style sheet, cannot be null
    * @param sourceFile directory to look into if the style sheet
    *    is not in the rules directory
    * @return "compiled" form of the style sheet
    */
   protected synchronized Templates getTemplatesHelper(String stylesheet,
                                                       File sourceFile)
      throws XMException
   {
      try
      {
         Templates templates = (Templates)templatesMap.get(stylesheet);
         if(templates == null)
         {
            Source source = null;
            if(sourceFile != null)
            {
               File sourceDir = sourceFile.getParentFile();
               referenceResolver.setDocsDir(sourceFile.getParentFile());
               source = referenceResolver.resolve(stylesheet,sourceDir.getPath());
            }
            else
               source = referenceResolver.resolve(stylesheet,null);
            boolean store = false;
            if(source == null)
               // if it does not find the style sheet in the rules
               // directory, assumes the stylesheet is a system id
               source = new StreamSource(stylesheet);
            else
               store = true;
            try
            {
               templates = factory.newTemplates(source);
            }
            catch(TransformerConfigurationException e)
            {


/*               System.out.println(e.getLocationAsString());
         if(e.getException() != null)
         {
            if(((TransformerException)e.getException()).getException() != null)
               System.out.println(((SAXParseException)((TransformerException)e.getException()).getException()).getLineNumber());
            else
               System.out.println(e.getException().toString());
         }*/


               messenger.fatal(new XMException(e));
            }
            // if it does not originate from the rules directory,
            // the style sheet may be relative to the document directory
            // to simplify cache management, I do not cache those
            if(store)
               templatesMap.put(stylesheet,templates);
         }
         return templates;
      }
      catch(TransformerException e)
      {
         throw new XMException(e);
      }
   }

   /**
    * retrieves the default templates
    * @return "compiled" version of the default Templates
    */
   public synchronized Templates getDefaultTemplates()
   {
      return (Templates)templatesMap.get("default.xsl");
   }

   /**
    * Returns a Transformer object for to the style sheet.
    * This is equivallent to calling newTransformer() on the result
    * of getTemplates() and setting the URIResolver with the result
    * of getURIResolver().
    * @param stylesheet URI for the style sheet, may be null (to
    *    access the default style sheet)
    * @param sourceFile directory to look into if the style sheet
    *    is not in the rules directory
    * @return Transformer object for the style sheet
    */
   public Transformer getTransformer(String stylesheet,File sourceFile)
      throws XMException
   {
      try
      {
         Templates templates = getTemplates(stylesheet,sourceFile);
         if(templates == null)
            return null;
         Transformer transformer = templates.newTransformer();
         if(sourceFile != null)
            referenceResolver.setDocsDir(sourceFile.getParentFile());
         transformer.setURIResolver(referenceResolver);
         return transformer;
      }
      catch(TransformerConfigurationException e)
      {
         messenger.fatal(new XMException(e));
         return null;
      }
   }

   /**
    * Returns a Transformer object for to the default style sheet.
    * This is equivallent to calling newTransformer() on the result
    * of getDefaultTemplates() and setting the URIResolver with the
    * result of getURIResolver().
    * @return Transformer object for the default style sheet
    */
   public Transformer getDefaultTransformer()
      throws XMException
   {
      try
      {
         Templates templates = getDefaultTemplates();
         Transformer transformer = templates.newTransformer();
         referenceResolver.setDocsDir(null);
         transformer.setURIResolver(referenceResolver);
         return transformer;
      }
      catch(TransformerConfigurationException e)
      {
         messenger.fatal(new XMException(e));
         return null;
      }
   }

   /**
    * Returns a TransformerHandler object for to the style sheet.
    * This is equivallent to calling newTransformerHandler() on the
    * result of getTemplates() and setting the URIResolver with the
    * result of getURIResolver().
    * @param stylesheet URI for the style sheet, may be null (to
    *    access the default style sheet)
    * @param sourceFile directory to look into if the style sheet
    *    is not in the rules directory
    * @return TransformerHandler object for the style sheet
    */
   protected TransformerHandler getTransformerHandler(String stylesheet,
                                                      File sourceFile)
      throws XMException
   {
      try
      {
         Templates templates = getTemplates(stylesheet,sourceFile);
         TransformerHandler handler = factory.newTransformerHandler(templates);
         referenceResolver.setDocsDir(sourceFile.getParentFile());
         handler.getTransformer().setURIResolver(referenceResolver);
         return handler;
      }
      catch(TransformerConfigurationException e)
      {
         messenger.fatal(new XMException(e));
         return null;
      }
   }

   /**
    * Returns a TransformerHandler object for to the default style sheet.
    * This is equivallent to calling newTransformerHandler() on the result
    * of getDefaultTemplates() and setting the URIResolver with the
    * result of getURIResolver().
    * @return TransformerHandlker object for the default style sheet
    */
   public TransformerHandler getDefaultTransformerHandler()
      throws XMException
   {
      try
      {
         Templates templates = getDefaultTemplates();
         TransformerHandler handler = factory.newTransformerHandler(templates);
         referenceResolver.setDocsDir(null);
         handler.getTransformer().setURIResolver(referenceResolver);
         return handler;
      }
      catch(TransformerConfigurationException e)
      {
         messenger.fatal(new XMException(e));
         return null;
      }
   }

   /**
    * Returns an URIResolver that properly loads style sheets
    * from the rules directory.
    * @return URIResolver that known about the rules directory
    */
   public URIResolver getURIResolver()
   {
      return referenceResolver;
   }
}
