/**
 * WalkFilter processes xm:Directory elements. It replaces
 * them by the content of a directory, turned into an XML document.
 *
 * @author <a href="http://www.marchal.com">Beno&#238;t Marchal</a>
 */

// Copyright 2001, Benot Marchal.

// pending issue: it filters out the characters & whitespaces
// in an xm:Directory element; what about the entities
// and processing instructions?

package org.ananas.xm;

import java.io.*;
import org.xml.sax.*;
import java.util.Date;
import org.xml.sax.helpers.*;
import java.text.SimpleDateFormat;
import org.ananas.util.NotImplementedException;

public class WalkFilter
   extends XMLFilterImpl
{
   /**
    * namespace URI for WalkFilter
    */
   public final static String namespaceURI =
      "http://www.ananas.org/2001/XM/Walk";

   /**
    * required to read a directory as an XML document
    */
   protected DirectoryReader directoryReader;

   /**
    * output messages on this messenger
    */
   protected Messenger messenger = null;

   /**
    * used to make sure that directories are not nested, one
    * in the other
    */
   protected boolean inDirectory;

   /**
    * the source file, the caller tries to set it (through a call to
    * setProperty()) so that we can mark the document
    */
   protected File sourceFile = null;

   /**
    * creates a new DirectoryFilter
    * @param messenger messenger to report progress and error messages to
    */
   public WalkFilter(Messenger messenger)
   {
      super();
      this.messenger = messenger;
      directoryReader = new DirectoryReader(messenger,true);
   }

   /**
    * creates a new DirectoryFilter
    * @param parent the filter's parent
    * @param messenger messenger to report progress and error messages to
    */
   public WalkFilter(XMLReader parent,Messenger messenger)
   {
      super(parent);
      this.messenger = messenger;
      directoryReader = new DirectoryReader(messenger,true);
   }

   /**
    * sets the ContentHandler that this filter reports events to
    * @param contentHandler the new ContentHandler
    */
   public void setContentHandler(ContentHandler contentHandler)
   {
      super.setContentHandler(contentHandler);
      directoryReader.setContentHandler(contentHandler);
   }

   /**
    * called by the parser when it starts reading a document
    * reset the filter
    */
   public void startDocument()
      throws SAXException
   {
      inDirectory = false;
      super.startDocument();
   }

   /**
    * a new element has been found
    * most elements are passed unmodified to the next filter but
    * xm:Directory is replaced by a directory content
    */
   public void startElement(String namespaceURI,
                            String localName,
                            String qName,
                            Attributes atts)
      throws SAXException
   {
      if(!namespaceURI.equals(this.namespaceURI))
         super.startElement(namespaceURI,localName,qName,atts);
      else
         try
         {
            if(localName.equals("Directory"))
            {
               if(inDirectory)
                  messenger.error(new XMException(Resources.getString("nestedelement"),
                                                  new Object[] { qName }));
               String dir = atts.getValue("dir");
               if(dir == null || dir.trim().length() == 0)
                  messenger.error(new XMException(Resources.getString("dirrequired"),
                                  new Object[] { qName }));
               String stValue = atts.getValue("markSource");
               boolean markSource = stValue != null && stValue.equalsIgnoreCase("true");
               if(markSource && sourceFile == null)
                  throw new NotImplementedException("Caller has not set the sourceFile but the document wants it marked.");
               File sourceDir = sourceFile != null ? sourceFile.getParentFile() : null,
                    dirFile = null;
               if(sourceDir != null)
                  dirFile = new File(sourceDir,dir);
               else
                  dirFile = new File(dir);
               directoryReader.read(dirFile,markSource ? sourceFile : null);
               inDirectory = true;
            }
         }
         catch(XMException e)
         {
            throw new SAXException(e);
         }
   }

   /**
    * end of the element
    * most elements are passed unmodified to the next filter but
    * xm:Directory is replaced by the directory content, the
    * replacement effectivly happens in the startElement()
    */
   public void endElement(String namespaceURI,
                          String localName,
                          String qName)
      throws SAXException
   {
      if(!namespaceURI.equals(this.namespaceURI))
         super.endElement(namespaceURI,localName,qName);
      else if(localName.equals("Directory"))
         inDirectory = false;
   }

   /**
    * character found
    * characters are passed unmodified to the next filter except
    * if they are within an xm:Directory
    */
   public void characters(char[] ch,int start,int length)
      throws SAXException
   {
      if(!inDirectory)
         super.characters(ch,start,length);
   }

   /**
    * whitespaces found
    * characters are passed unmodified to the next filter except
    * if they are within an xm:Directory
    */
   public void ignorableWhitespace(char[] ch,int start,int length)
      throws SAXException
   {
      if(!inDirectory)
         super.ignorableWhitespace(ch,start,length);
   }

   /**
    * the caller sets a property, we intercept the sourceFile
    * in which case it expects a File object
    * @param name property name (an URI)
    * @param value property value
    * @throw SAXNotRecognizedException if the URI is unknown
    * @throw SAXNotSupportedException if it cannot set the property
    *   this method only throws an exception if the value is not
    *   a File object but the parent may throw other exceptions
    */
   public void setProperty(String name,Object value)
      throws SAXNotRecognizedException, SAXNotSupportedException
   {
      if(name.equals(Mover.SOURCE_FILE))
         if(value instanceof File)
            sourceFile = (File)value;
         else
            throw new SAXNotSupportedException("Value parameter must be a File object");
      else
         super.setProperty(name,value);
   }

   /**
    * returns a property, we intercept the sourceFile
    * @param name property name (an URI)
    * @return property value
    * @throw SAXNotRecognizedException if the URI is unknown
    * @throw SAXNotSupportedException if it cannot read the property
    */
   public Object getProperty(String name)
      throws SAXNotRecognizedException, SAXNotSupportedException
   {
      if(name.equals(Mover.SOURCE_FILE))
         return sourceFile;
      else
         return super.getProperty(name);
   }
}
