package org.ananas.xm.eclipse;

import java.io.File;
import org.ananas.xm.*;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Implements a Messenger that forwards its progress information to
 * an Eclipse IProgressMonitor.
 * Note that it ignores every message but progress... You will want
 * to pair this Messenger with another display for errors.
 * Beware that you have to explicitely call the done() method
 * because Monitor needs to be notified when processing is done
 * but Messenger has no such method equivallent.
 * 
 * @author bmarchal
 */
public class MessengerMonitor
   implements Messenger
{
   /**
    * The IProgressMonitor on which to report status.
    */
   private IProgressMonitor monitor;

   /**
    * Creates a new MessengerMonitor. It calls beginTask() with
    * the given task name and an unknown total work.
    * @param monitor monitor to forward progress messages to
    * @param taskName task name to report to the monitor
    */
   public MessengerMonitor(IProgressMonitor monitor,String taskName)
   {
   	  this.monitor = monitor;
      monitor.beginTask(taskName,IProgressMonitor.UNKNOWN);
   }

   /**
    * the message is ignored
    * @see org.ananas.xm.Messenger#begin(String, String)
    */
   public void begin(String source,String target)
   {
   }

   /**
    * Tell the monitor that the processing is complete.
    * @see org.ananas.xm.Messenger#end()
    */
   public void end()
   {
      monitor.done();
   }

   /**
    * passed to the IProgressMonitor
    * @see org.ananas.xm.Messenger#progress(java.io.File, java.io.File)
    */
   public boolean progress(File sourceFile,File resultFile)
      throws XMException
   {
   	  monitor.worked(IProgressMonitor.UNKNOWN);
      return !monitor.isCanceled();
   }

   /**
    * ignored
    * @see org.ananas.xm.Messenger#error(org.ananas.xm.XMException)
    */
   public void error(XMException e)
      throws XMException
   {
   }

   /**
    * ignore
    * @see org.ananas.xm.Messenger#fatal(org.ananas.xm.XMException)
    */
   public void fatal(XMException e)
      throws XMException
   {
   }

   /**
    * ignore
    * @see org.ananas.xm.Messenger#warning(org.ananas.xm.XMException)
    */
   public void warning(XMException e)
      throws XMException
   {
   }

   /**
    * ignore
    * @see org.ananas.xm.Messenger#info(java.lang.String)
    */
   public void info(String msg)
      throws XMException
   {
   }

   /**
    * ignore
    * @see org.ananas.xm.Messenger#info(java.lang.String, java.lang.Object)
    */
   public void info(String pattern,Object[] arguments)
      throws XMException
   {
   }
}