package org.ananas.xm.eclipse;

// I have changed the package name through the Refactor menu...

import java.io.File;

import java.util.*;
import org.ananas.xm.*;

/**
 * This class implements a proxy mechanism for Messengers: it is itself
 * a Messenger and it forwards requests to a list of Messengers.
 * 
 * @author bmarchal
 */
public class MessengerProxy
   implements Messenger
{
   /**
    * Holds the various Messengers this proxy forwards to.
    */
   protected List messengers = new ArrayList();

   /**
    * Adds a new Messenger to the proxy. From now on, events will be forwarded
    * to the Messenger as well.
    * @param messenger Messenger to add to the proxy forwarding
    */
   public synchronized void addMessenger(Messenger messenger)
   {
      messengers.add(messenger);
   }

   /**
    * Removes a Messenger from the proxy. From now on, it won't receive
    * the events. 
    * @param messenger Messenger to remove from the proxy forwarding
    */
   public synchronized void removeMessenger(Messenger messenger)
   {
      messengers.remove(messenger);
   }

   /**
    * Forwards the event to the list of Messengers. Will throw an exception
    * if at least one of the Messengers in the list throws an exception.
    * The exception it throws encapsulates one of the exceptions it has received,
    * application should not count on retrieving a specific exception. 
    * @see org.ananas.xm.Messenger#error(org.ananas.xm.XMException)
    */
   public synchronized void error(XMException e)
      throws XMException
   {
      Iterator iterator = messengers.iterator();
      XMException caught = null;
      while(iterator.hasNext())
         try
         {
            ((Messenger)iterator.next()).error(e);
         }
         catch(XMException x)
         {
         	caught = x;
         }
      if(caught != null)
         throw new XMException(caught);
   }

   /**
    * Forwards the event to the list of Messengers. Will throw an exception
    * if at least one of the Messengers in the list throws an exception.
    * The exception it throws encapsulates one of the exceptions it has received,
    * application should not count on retrieving a specific exception. 
    * @see org.ananas.xm.Messenger#fatal(org.ananas.xm.XMException)
    */
   public synchronized void fatal(XMException e)
      throws XMException
   {
      Iterator iterator = messengers.iterator();
      XMException caught = null;
      while(iterator.hasNext())
         try
         {
            ((Messenger)iterator.next()).fatal(e);
         }
         catch(XMException x)
         {
         	caught = x;
         }
      if(caught != null)
         throw new XMException(caught);
   }

   /**
    * Forwards the event to the list of Messengers. Will throw an exception
    * if at least one of the Messengers in the list throws an exception.
    * The exception it throws encapsulates one of the exceptions it has received,
    * application should not count on retrieving a specific exception. 
    * @see org.ananas.xm.Messenger#warning(org.ananas.xm.XMException)
    */
   public synchronized void warning(XMException e)
      throws XMException
   {
      Iterator iterator = messengers.iterator();
      XMException caught = null;
      while(iterator.hasNext())
         try
         {
            ((Messenger)iterator.next()).warning(e);
         }
         catch(XMException x)
         {
         	caught = x;
         }
      if(caught != null)
         throw caught;
   }

   /**
    * Forwards the event to the list of Messengers. Will throw an exception
    * if at least one of the Messengers in the list throws an exception.
    * The exception it throws encapsulates one of the exceptions it has received,
    * application should not count on retrieving a specific exception.
    * @return false if one of the Messengers has returned false, true otherwise 
    * @see org.ananas.xm.Messenger#progress(java.io.File, java.io.File)
    */ 
   public boolean progress(File sourceFile,File resultFile)
      throws XMException
   {
      Iterator iterator = messengers.iterator();
      XMException caught = null;
      boolean doNext = true;
      while(iterator.hasNext())
         try
         {
            ((Messenger)iterator.next()).progress(sourceFile,resultFile);
            if(!((Messenger)iterator.next()).progress(sourceFile,resultFile))
               doNext = false;
         }
         catch(XMException x)
         {
         	caught = x;
         }
      if(caught != null)
         throw new XMException(caught);
      return doNext;
   }

   /**
    * Forwards the event to the list of Messengers. Will throw an exception
    * if at least one of the Messengers in the list throws an exception.
    * The exception it throws encapsulates one of the exceptions it has received,
    * application should not count on retrieving a specific exception.
    * @see org.ananas.xm.Messenger#info(java.lang.String)
    */
	public void info(String msg)
	   throws XMException
	{
      Iterator iterator = messengers.iterator();
      XMException caught = null;
      while(iterator.hasNext())
         try
         {
            ((Messenger)iterator.next()).info(msg);
         }
         catch(XMException x)
         {
         	caught = x;
         }
      if(caught != null)
         throw new XMException(caught);
	}

   /**
    * Forwards the event to the list of Messengers. Will throw an exception
    * if at least one of the Messengers in the list throws an exception.
    * The exception it throws encapsulates one of the exceptions it has received,
    * application should not count on retrieving a specific exception.
    * @see org.ananas.xm.Messenger#info(java.lang.String, java.lang.Object)
    */
   public void info(String pattern,Object[] arguments)
      throws XMException
   {
      Iterator iterator = messengers.iterator();
      XMException caught = null;
      while(iterator.hasNext())
         try
         {
            ((Messenger)iterator.next()).info(pattern,arguments);
         }
         catch(XMException x)
         {
         	caught = x;
         }
      if(caught != null)
         throw new XMException(caught);
   }

   /**
    * Forwards the event to the list of Messengers. Will throw an exception
    * if at least one of the Messengers in the list throws an exception.
    * The exception it throws encapsulates one of the exceptions it has received,
    * application should not count on retrieving a specific exception.
    * @see org.ananas.xm.Messenger#begin(java.lang.String, java.lang.String)
    */
   public void begin(String source,String target)
      throws XMException
   {
      Iterator iterator = messengers.iterator();
      XMException caught = null;
      while(iterator.hasNext())
         try
         {
            ((Messenger)iterator.next()).begin(source,target);
         }
         catch(XMException x)
         {
         	caught = x;
         }
      if(caught != null)
         throw new XMException(caught);
   }

   /**
    * Forwards the event to the list of Messengers. Will throw an exception
    * if at least one of the Messengers in the list throws an exception.
    * The exception it throws encapsulates one of the exceptions it has received,
    * application should not count on retrieving a specific exception.
    * @see org.ananas.xm.Messenger#end()
    */
   public void end()
      throws XMException
   {
      Iterator iterator = messengers.iterator();
      XMException caught = null;
      while(iterator.hasNext())
         try
         {
            ((Messenger)iterator.next()).end();
         }
         catch(XMException x)
         {
         	caught = x;
         }
      if(caught != null)
         throw new XMException(caught);
   }
}