package org.ananas.xm.eclipse;

import java.io.*;
import java.util.*;
import org.ananas.xm.*;
import org.eclipse.ui.*;
import org.eclipse.ui.actions.*;
import org.eclipse.jface.action.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.core.resources.*;
import java.lang.reflect.InvocationTargetException;
import org.eclipse.jface.operation.IRunnableWithProgress;

/**
 * XMRunner executes XM when right-clicking on the project.
 * @author bmarchal
 */
public class XMRunner
   extends ActionDelegate
   implements IObjectActionDelegate
{
   /**
    * the resource that the user has selected in the UI
    */
   private IResource resource;
   
   /**
    * a pointer to the UI
    */
   private IWorkbenchPart part;

   /**
    * User selected our entry in the menu, it's time to do something:
    * run XM (in practice, it builds the project).
    * @param action action event details
    * @see org.eclipse.ui.IActionDelegate#run(IAction)
    */
   public void run(IAction action)
   {
   	  if(resource != null)
         try
         {
            ProgressMonitorDialog dialog = new ProgressMonitorDialog(part.getSite().getShell());
            dialog.run(false,true,new IRunnableWithProgress()
            {
               public void run(IProgressMonitor monitor)
               {
                  try
                  {
                     validateProjectNature(monitor); 
                     IProject project = resource.getProject();
                     ProjectProperties properties = new ProjectProperties(project);
                     project.build(properties.isBuild()
                                   ? IncrementalProjectBuilder.FULL_BUILD
                                   : IncrementalProjectBuilder.INCREMENTAL_BUILD,
                                   monitor);
                  }
                  catch(CoreException x)
                  {
                     reportError(x);
                  }
               }
            });
         }
         catch(InterruptedException x)
         {
            reportError(x);
         }
         catch(InvocationTargetException x)
         {
            reportError(x);
         }
   }
	
   /**
    * User has selected another resource.
    * @see org.eclipse.ui.IActionDelegate#selectionChanged(IAction, ISelection)
    */
   public void selectionChanged(IAction action,ISelection selection)
   {
      resource = null;
      if(selection instanceof IStructuredSelection)
      {
         IStructuredSelection structuredSelection = (IStructuredSelection)selection;
         if(structuredSelection.size() == 1)
         {
            Object selectedResource = structuredSelection.getFirstElement();
            if(selectedResource instanceof IResource)
               resource = (IResource)selectedResource;
         }
      }
   }

   /**
    * User has changed the active part in the UI.
    * @see org.eclipse.ui.IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
    */	
   public void setActivePart(IAction action,IWorkbenchPart targetPart)
   {
      part = targetPart;
   }

   /**
    * Displays an error that occured during the project creation.
    * @param x details on the error
    */
   private void reportError(Exception x)
   {
      if(part != null)
         ErrorDialog.openError(part.getSite().getShell(),
                               Resources.getString("eclipse.dialogtitle"),
                               Resources.getString("eclipse.launcherror"),
                               PluginTools.makeStatus(x));
      else
         x.printStackTrace();
   }

   /**
    * Makes sure that the project is of XM nature.
    * @param monitor progress monitor to report data on
    */
   public void validateProjectNature(IProgressMonitor monitor)
      throws CoreException
   {
      IProject project = resource.getProject();
      if(!project.hasNature(PluginConstants.NATURE_ID))
      {
         IProjectDescription description = project.getDescription();
         String[] old = description.getNatureIds(),
                  natures= new String[old.length + 1];
         System.arraycopy(old,0,natures,0,old.length);
         natures[old.length] = PluginConstants.NATURE_ID;
         description.setNatureIds(natures);
         project.setDescription(description,monitor);
      }
   }
}